// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.25.0
// 	protoc        v3.11.4
// source: acn.proto

package aea_aea_acn_v1_0_0

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type AcnMessage_StatusBody_StatusCodeEnum int32

const (
	// common (0x)
	AcnMessage_StatusBody_SUCCESS                   AcnMessage_StatusBody_StatusCodeEnum = 0
	AcnMessage_StatusBody_ERROR_UNSUPPORTED_VERSION AcnMessage_StatusBody_StatusCodeEnum = 1
	AcnMessage_StatusBody_ERROR_UNEXPECTED_PAYLOAD  AcnMessage_StatusBody_StatusCodeEnum = 2
	AcnMessage_StatusBody_ERROR_GENERIC             AcnMessage_StatusBody_StatusCodeEnum = 3
	AcnMessage_StatusBody_ERROR_DECODE              AcnMessage_StatusBody_StatusCodeEnum = 4
	// register (1x)
	AcnMessage_StatusBody_ERROR_WRONG_AGENT_ADDRESS AcnMessage_StatusBody_StatusCodeEnum = 10
	AcnMessage_StatusBody_ERROR_WRONG_PUBLIC_KEY    AcnMessage_StatusBody_StatusCodeEnum = 11
	AcnMessage_StatusBody_ERROR_INVALID_PROOF       AcnMessage_StatusBody_StatusCodeEnum = 12
	AcnMessage_StatusBody_ERROR_UNSUPPORTED_LEDGER  AcnMessage_StatusBody_StatusCodeEnum = 13
	// lookup & delivery (2x)
	AcnMessage_StatusBody_ERROR_UNKNOWN_AGENT_ADDRESS AcnMessage_StatusBody_StatusCodeEnum = 20
	AcnMessage_StatusBody_ERROR_AGENT_NOT_READY       AcnMessage_StatusBody_StatusCodeEnum = 21
)

// Enum value maps for AcnMessage_StatusBody_StatusCodeEnum.
var (
	AcnMessage_StatusBody_StatusCodeEnum_name = map[int32]string{
		0:  "SUCCESS",
		1:  "ERROR_UNSUPPORTED_VERSION",
		2:  "ERROR_UNEXPECTED_PAYLOAD",
		3:  "ERROR_GENERIC",
		4:  "ERROR_DECODE",
		10: "ERROR_WRONG_AGENT_ADDRESS",
		11: "ERROR_WRONG_PUBLIC_KEY",
		12: "ERROR_INVALID_PROOF",
		13: "ERROR_UNSUPPORTED_LEDGER",
		20: "ERROR_UNKNOWN_AGENT_ADDRESS",
		21: "ERROR_AGENT_NOT_READY",
	}
	AcnMessage_StatusBody_StatusCodeEnum_value = map[string]int32{
		"SUCCESS":                     0,
		"ERROR_UNSUPPORTED_VERSION":   1,
		"ERROR_UNEXPECTED_PAYLOAD":    2,
		"ERROR_GENERIC":               3,
		"ERROR_DECODE":                4,
		"ERROR_WRONG_AGENT_ADDRESS":   10,
		"ERROR_WRONG_PUBLIC_KEY":      11,
		"ERROR_INVALID_PROOF":         12,
		"ERROR_UNSUPPORTED_LEDGER":    13,
		"ERROR_UNKNOWN_AGENT_ADDRESS": 20,
		"ERROR_AGENT_NOT_READY":       21,
	}
)

func (x AcnMessage_StatusBody_StatusCodeEnum) Enum() *AcnMessage_StatusBody_StatusCodeEnum {
	p := new(AcnMessage_StatusBody_StatusCodeEnum)
	*p = x
	return p
}

func (x AcnMessage_StatusBody_StatusCodeEnum) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (AcnMessage_StatusBody_StatusCodeEnum) Descriptor() protoreflect.EnumDescriptor {
	return file_acn_proto_enumTypes[0].Descriptor()
}

func (AcnMessage_StatusBody_StatusCodeEnum) Type() protoreflect.EnumType {
	return &file_acn_proto_enumTypes[0]
}

func (x AcnMessage_StatusBody_StatusCodeEnum) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use AcnMessage_StatusBody_StatusCodeEnum.Descriptor instead.
func (AcnMessage_StatusBody_StatusCodeEnum) EnumDescriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 1, 0}
}

type AcnMessage struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	// Types that are assignable to Performative:
	//	*AcnMessage_AeaEnvelope
	//	*AcnMessage_LookupRequest
	//	*AcnMessage_LookupResponse
	//	*AcnMessage_Register
	//	*AcnMessage_Status
	Performative isAcnMessage_Performative `protobuf_oneof:"performative"`
}

func (x *AcnMessage) Reset() {
	*x = AcnMessage{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[0]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage) ProtoMessage() {}

func (x *AcnMessage) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[0]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage.ProtoReflect.Descriptor instead.
func (*AcnMessage) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0}
}

func (m *AcnMessage) GetPerformative() isAcnMessage_Performative {
	if m != nil {
		return m.Performative
	}
	return nil
}

func (x *AcnMessage) GetAeaEnvelope() *AcnMessage_Aea_Envelope_Performative {
	if x, ok := x.GetPerformative().(*AcnMessage_AeaEnvelope); ok {
		return x.AeaEnvelope
	}
	return nil
}

func (x *AcnMessage) GetLookupRequest() *AcnMessage_Lookup_Request_Performative {
	if x, ok := x.GetPerformative().(*AcnMessage_LookupRequest); ok {
		return x.LookupRequest
	}
	return nil
}

func (x *AcnMessage) GetLookupResponse() *AcnMessage_Lookup_Response_Performative {
	if x, ok := x.GetPerformative().(*AcnMessage_LookupResponse); ok {
		return x.LookupResponse
	}
	return nil
}

func (x *AcnMessage) GetRegister() *AcnMessage_Register_Performative {
	if x, ok := x.GetPerformative().(*AcnMessage_Register); ok {
		return x.Register
	}
	return nil
}

func (x *AcnMessage) GetStatus() *AcnMessage_Status_Performative {
	if x, ok := x.GetPerformative().(*AcnMessage_Status); ok {
		return x.Status
	}
	return nil
}

type isAcnMessage_Performative interface {
	isAcnMessage_Performative()
}

type AcnMessage_AeaEnvelope struct {
	AeaEnvelope *AcnMessage_Aea_Envelope_Performative `protobuf:"bytes,5,opt,name=aea_envelope,json=aeaEnvelope,proto3,oneof"`
}

type AcnMessage_LookupRequest struct {
	LookupRequest *AcnMessage_Lookup_Request_Performative `protobuf:"bytes,6,opt,name=lookup_request,json=lookupRequest,proto3,oneof"`
}

type AcnMessage_LookupResponse struct {
	LookupResponse *AcnMessage_Lookup_Response_Performative `protobuf:"bytes,7,opt,name=lookup_response,json=lookupResponse,proto3,oneof"`
}

type AcnMessage_Register struct {
	Register *AcnMessage_Register_Performative `protobuf:"bytes,8,opt,name=register,proto3,oneof"`
}

type AcnMessage_Status struct {
	Status *AcnMessage_Status_Performative `protobuf:"bytes,9,opt,name=status,proto3,oneof"`
}

func (*AcnMessage_AeaEnvelope) isAcnMessage_Performative() {}

func (*AcnMessage_LookupRequest) isAcnMessage_Performative() {}

func (*AcnMessage_LookupResponse) isAcnMessage_Performative() {}

func (*AcnMessage_Register) isAcnMessage_Performative() {}

func (*AcnMessage_Status) isAcnMessage_Performative() {}

// Custom Types
type AcnMessage_AgentRecord struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	ServiceId     string `protobuf:"bytes,1,opt,name=service_id,json=serviceId,proto3" json:"service_id,omitempty"`
	LedgerId      string `protobuf:"bytes,2,opt,name=ledger_id,json=ledgerId,proto3" json:"ledger_id,omitempty"`
	Address       string `protobuf:"bytes,3,opt,name=address,proto3" json:"address,omitempty"`
	PublicKey     string `protobuf:"bytes,4,opt,name=public_key,json=publicKey,proto3" json:"public_key,omitempty"`
	PeerPublicKey string `protobuf:"bytes,5,opt,name=peer_public_key,json=peerPublicKey,proto3" json:"peer_public_key,omitempty"`
	Signature     string `protobuf:"bytes,6,opt,name=signature,proto3" json:"signature,omitempty"`
	NotBefore     string `protobuf:"bytes,7,opt,name=not_before,json=notBefore,proto3" json:"not_before,omitempty"`
	NotAfter      string `protobuf:"bytes,8,opt,name=not_after,json=notAfter,proto3" json:"not_after,omitempty"`
}

func (x *AcnMessage_AgentRecord) Reset() {
	*x = AcnMessage_AgentRecord{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[1]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage_AgentRecord) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage_AgentRecord) ProtoMessage() {}

func (x *AcnMessage_AgentRecord) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[1]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage_AgentRecord.ProtoReflect.Descriptor instead.
func (*AcnMessage_AgentRecord) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 0}
}

func (x *AcnMessage_AgentRecord) GetServiceId() string {
	if x != nil {
		return x.ServiceId
	}
	return ""
}

func (x *AcnMessage_AgentRecord) GetLedgerId() string {
	if x != nil {
		return x.LedgerId
	}
	return ""
}

func (x *AcnMessage_AgentRecord) GetAddress() string {
	if x != nil {
		return x.Address
	}
	return ""
}

func (x *AcnMessage_AgentRecord) GetPublicKey() string {
	if x != nil {
		return x.PublicKey
	}
	return ""
}

func (x *AcnMessage_AgentRecord) GetPeerPublicKey() string {
	if x != nil {
		return x.PeerPublicKey
	}
	return ""
}

func (x *AcnMessage_AgentRecord) GetSignature() string {
	if x != nil {
		return x.Signature
	}
	return ""
}

func (x *AcnMessage_AgentRecord) GetNotBefore() string {
	if x != nil {
		return x.NotBefore
	}
	return ""
}

func (x *AcnMessage_AgentRecord) GetNotAfter() string {
	if x != nil {
		return x.NotAfter
	}
	return ""
}

type AcnMessage_StatusBody struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Code AcnMessage_StatusBody_StatusCodeEnum `protobuf:"varint,1,opt,name=code,proto3,enum=aea.aea.acn.v1_0_0.AcnMessage_StatusBody_StatusCodeEnum" json:"code,omitempty"`
	Msgs []string                             `protobuf:"bytes,2,rep,name=msgs,proto3" json:"msgs,omitempty"`
}

func (x *AcnMessage_StatusBody) Reset() {
	*x = AcnMessage_StatusBody{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[2]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage_StatusBody) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage_StatusBody) ProtoMessage() {}

func (x *AcnMessage_StatusBody) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[2]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage_StatusBody.ProtoReflect.Descriptor instead.
func (*AcnMessage_StatusBody) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 1}
}

func (x *AcnMessage_StatusBody) GetCode() AcnMessage_StatusBody_StatusCodeEnum {
	if x != nil {
		return x.Code
	}
	return AcnMessage_StatusBody_SUCCESS
}

func (x *AcnMessage_StatusBody) GetMsgs() []string {
	if x != nil {
		return x.Msgs
	}
	return nil
}

// Performatives and contents
type AcnMessage_Register_Performative struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Record *AcnMessage_AgentRecord `protobuf:"bytes,1,opt,name=record,proto3" json:"record,omitempty"`
}

func (x *AcnMessage_Register_Performative) Reset() {
	*x = AcnMessage_Register_Performative{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[3]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage_Register_Performative) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage_Register_Performative) ProtoMessage() {}

func (x *AcnMessage_Register_Performative) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[3]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage_Register_Performative.ProtoReflect.Descriptor instead.
func (*AcnMessage_Register_Performative) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 2}
}

func (x *AcnMessage_Register_Performative) GetRecord() *AcnMessage_AgentRecord {
	if x != nil {
		return x.Record
	}
	return nil
}

type AcnMessage_Lookup_Request_Performative struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	AgentAddress string `protobuf:"bytes,1,opt,name=agent_address,json=agentAddress,proto3" json:"agent_address,omitempty"`
}

func (x *AcnMessage_Lookup_Request_Performative) Reset() {
	*x = AcnMessage_Lookup_Request_Performative{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[4]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage_Lookup_Request_Performative) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage_Lookup_Request_Performative) ProtoMessage() {}

func (x *AcnMessage_Lookup_Request_Performative) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[4]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage_Lookup_Request_Performative.ProtoReflect.Descriptor instead.
func (*AcnMessage_Lookup_Request_Performative) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 3}
}

func (x *AcnMessage_Lookup_Request_Performative) GetAgentAddress() string {
	if x != nil {
		return x.AgentAddress
	}
	return ""
}

type AcnMessage_Lookup_Response_Performative struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Record *AcnMessage_AgentRecord `protobuf:"bytes,1,opt,name=record,proto3" json:"record,omitempty"`
}

func (x *AcnMessage_Lookup_Response_Performative) Reset() {
	*x = AcnMessage_Lookup_Response_Performative{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[5]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage_Lookup_Response_Performative) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage_Lookup_Response_Performative) ProtoMessage() {}

func (x *AcnMessage_Lookup_Response_Performative) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[5]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage_Lookup_Response_Performative.ProtoReflect.Descriptor instead.
func (*AcnMessage_Lookup_Response_Performative) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 4}
}

func (x *AcnMessage_Lookup_Response_Performative) GetRecord() *AcnMessage_AgentRecord {
	if x != nil {
		return x.Record
	}
	return nil
}

type AcnMessage_Aea_Envelope_Performative struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Envelope []byte                  `protobuf:"bytes,1,opt,name=envelope,proto3" json:"envelope,omitempty"`
	Record   *AcnMessage_AgentRecord `protobuf:"bytes,2,opt,name=record,proto3" json:"record,omitempty"`
}

func (x *AcnMessage_Aea_Envelope_Performative) Reset() {
	*x = AcnMessage_Aea_Envelope_Performative{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[6]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage_Aea_Envelope_Performative) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage_Aea_Envelope_Performative) ProtoMessage() {}

func (x *AcnMessage_Aea_Envelope_Performative) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[6]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage_Aea_Envelope_Performative.ProtoReflect.Descriptor instead.
func (*AcnMessage_Aea_Envelope_Performative) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 5}
}

func (x *AcnMessage_Aea_Envelope_Performative) GetEnvelope() []byte {
	if x != nil {
		return x.Envelope
	}
	return nil
}

func (x *AcnMessage_Aea_Envelope_Performative) GetRecord() *AcnMessage_AgentRecord {
	if x != nil {
		return x.Record
	}
	return nil
}

type AcnMessage_Status_Performative struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Body *AcnMessage_StatusBody `protobuf:"bytes,1,opt,name=body,proto3" json:"body,omitempty"`
}

func (x *AcnMessage_Status_Performative) Reset() {
	*x = AcnMessage_Status_Performative{}
	if protoimpl.UnsafeEnabled {
		mi := &file_acn_proto_msgTypes[7]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *AcnMessage_Status_Performative) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AcnMessage_Status_Performative) ProtoMessage() {}

func (x *AcnMessage_Status_Performative) ProtoReflect() protoreflect.Message {
	mi := &file_acn_proto_msgTypes[7]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AcnMessage_Status_Performative.ProtoReflect.Descriptor instead.
func (*AcnMessage_Status_Performative) Descriptor() ([]byte, []int) {
	return file_acn_proto_rawDescGZIP(), []int{0, 6}
}

func (x *AcnMessage_Status_Performative) GetBody() *AcnMessage_StatusBody {
	if x != nil {
		return x.Body
	}
	return nil
}

var File_acn_proto protoreflect.FileDescriptor

var file_acn_proto_rawDesc = []byte{
	0x0a, 0x09, 0x61, 0x63, 0x6e, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x12, 0x12, 0x61, 0x65, 0x61,
	0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f, 0x30, 0x22,
	0xea, 0x0c, 0x0a, 0x0a, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x12, 0x5d,
	0x0a, 0x0c, 0x61, 0x65, 0x61, 0x5f, 0x65, 0x6e, 0x76, 0x65, 0x6c, 0x6f, 0x70, 0x65, 0x18, 0x05,
	0x20, 0x01, 0x28, 0x0b, 0x32, 0x38, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61,
	0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f, 0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73,
	0x73, 0x61, 0x67, 0x65, 0x2e, 0x41, 0x65, 0x61, 0x5f, 0x45, 0x6e, 0x76, 0x65, 0x6c, 0x6f, 0x70,
	0x65, 0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69, 0x76, 0x65, 0x48, 0x00,
	0x52, 0x0b, 0x61, 0x65, 0x61, 0x45, 0x6e, 0x76, 0x65, 0x6c, 0x6f, 0x70, 0x65, 0x12, 0x63, 0x0a,
	0x0e, 0x6c, 0x6f, 0x6f, 0x6b, 0x75, 0x70, 0x5f, 0x72, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x18,
	0x06, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x3a, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e,
	0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f, 0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65,
	0x73, 0x73, 0x61, 0x67, 0x65, 0x2e, 0x4c, 0x6f, 0x6f, 0x6b, 0x75, 0x70, 0x5f, 0x52, 0x65, 0x71,
	0x75, 0x65, 0x73, 0x74, 0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69, 0x76,
	0x65, 0x48, 0x00, 0x52, 0x0d, 0x6c, 0x6f, 0x6f, 0x6b, 0x75, 0x70, 0x52, 0x65, 0x71, 0x75, 0x65,
	0x73, 0x74, 0x12, 0x66, 0x0a, 0x0f, 0x6c, 0x6f, 0x6f, 0x6b, 0x75, 0x70, 0x5f, 0x72, 0x65, 0x73,
	0x70, 0x6f, 0x6e, 0x73, 0x65, 0x18, 0x07, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x3b, 0x2e, 0x61, 0x65,
	0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f, 0x30,
	0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x2e, 0x4c, 0x6f, 0x6f, 0x6b,
	0x75, 0x70, 0x5f, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x5f, 0x50, 0x65, 0x72, 0x66,
	0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69, 0x76, 0x65, 0x48, 0x00, 0x52, 0x0e, 0x6c, 0x6f, 0x6f, 0x6b,
	0x75, 0x70, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12, 0x52, 0x0a, 0x08, 0x72, 0x65,
	0x67, 0x69, 0x73, 0x74, 0x65, 0x72, 0x18, 0x08, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x34, 0x2e, 0x61,
	0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f,
	0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x2e, 0x52, 0x65, 0x67,
	0x69, 0x73, 0x74, 0x65, 0x72, 0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69,
	0x76, 0x65, 0x48, 0x00, 0x52, 0x08, 0x72, 0x65, 0x67, 0x69, 0x73, 0x74, 0x65, 0x72, 0x12, 0x4c,
	0x0a, 0x06, 0x73, 0x74, 0x61, 0x74, 0x75, 0x73, 0x18, 0x09, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x32,
	0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f,
	0x30, 0x5f, 0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x2e, 0x53,
	0x74, 0x61, 0x74, 0x75, 0x73, 0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69,
	0x76, 0x65, 0x48, 0x00, 0x52, 0x06, 0x73, 0x74, 0x61, 0x74, 0x75, 0x73, 0x1a, 0x84, 0x02, 0x0a,
	0x0b, 0x41, 0x67, 0x65, 0x6e, 0x74, 0x52, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x12, 0x1d, 0x0a, 0x0a,
	0x73, 0x65, 0x72, 0x76, 0x69, 0x63, 0x65, 0x5f, 0x69, 0x64, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09,
	0x52, 0x09, 0x73, 0x65, 0x72, 0x76, 0x69, 0x63, 0x65, 0x49, 0x64, 0x12, 0x1b, 0x0a, 0x09, 0x6c,
	0x65, 0x64, 0x67, 0x65, 0x72, 0x5f, 0x69, 0x64, 0x18, 0x02, 0x20, 0x01, 0x28, 0x09, 0x52, 0x08,
	0x6c, 0x65, 0x64, 0x67, 0x65, 0x72, 0x49, 0x64, 0x12, 0x18, 0x0a, 0x07, 0x61, 0x64, 0x64, 0x72,
	0x65, 0x73, 0x73, 0x18, 0x03, 0x20, 0x01, 0x28, 0x09, 0x52, 0x07, 0x61, 0x64, 0x64, 0x72, 0x65,
	0x73, 0x73, 0x12, 0x1d, 0x0a, 0x0a, 0x70, 0x75, 0x62, 0x6c, 0x69, 0x63, 0x5f, 0x6b, 0x65, 0x79,
	0x18, 0x04, 0x20, 0x01, 0x28, 0x09, 0x52, 0x09, 0x70, 0x75, 0x62, 0x6c, 0x69, 0x63, 0x4b, 0x65,
	0x79, 0x12, 0x26, 0x0a, 0x0f, 0x70, 0x65, 0x65, 0x72, 0x5f, 0x70, 0x75, 0x62, 0x6c, 0x69, 0x63,
	0x5f, 0x6b, 0x65, 0x79, 0x18, 0x05, 0x20, 0x01, 0x28, 0x09, 0x52, 0x0d, 0x70, 0x65, 0x65, 0x72,
	0x50, 0x75, 0x62, 0x6c, 0x69, 0x63, 0x4b, 0x65, 0x79, 0x12, 0x1c, 0x0a, 0x09, 0x73, 0x69, 0x67,
	0x6e, 0x61, 0x74, 0x75, 0x72, 0x65, 0x18, 0x06, 0x20, 0x01, 0x28, 0x09, 0x52, 0x09, 0x73, 0x69,
	0x67, 0x6e, 0x61, 0x74, 0x75, 0x72, 0x65, 0x12, 0x1d, 0x0a, 0x0a, 0x6e, 0x6f, 0x74, 0x5f, 0x62,
	0x65, 0x66, 0x6f, 0x72, 0x65, 0x18, 0x07, 0x20, 0x01, 0x28, 0x09, 0x52, 0x09, 0x6e, 0x6f, 0x74,
	0x42, 0x65, 0x66, 0x6f, 0x72, 0x65, 0x12, 0x1b, 0x0a, 0x09, 0x6e, 0x6f, 0x74, 0x5f, 0x61, 0x66,
	0x74, 0x65, 0x72, 0x18, 0x08, 0x20, 0x01, 0x28, 0x09, 0x52, 0x08, 0x6e, 0x6f, 0x74, 0x41, 0x66,
	0x74, 0x65, 0x72, 0x1a, 0x9e, 0x03, 0x0a, 0x0a, 0x53, 0x74, 0x61, 0x74, 0x75, 0x73, 0x42, 0x6f,
	0x64, 0x79, 0x12, 0x4c, 0x0a, 0x04, 0x63, 0x6f, 0x64, 0x65, 0x18, 0x01, 0x20, 0x01, 0x28, 0x0e,
	0x32, 0x38, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e, 0x2e, 0x76,
	0x31, 0x5f, 0x30, 0x5f, 0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65,
	0x2e, 0x53, 0x74, 0x61, 0x74, 0x75, 0x73, 0x42, 0x6f, 0x64, 0x79, 0x2e, 0x53, 0x74, 0x61, 0x74,
	0x75, 0x73, 0x43, 0x6f, 0x64, 0x65, 0x45, 0x6e, 0x75, 0x6d, 0x52, 0x04, 0x63, 0x6f, 0x64, 0x65,
	0x12, 0x12, 0x0a, 0x04, 0x6d, 0x73, 0x67, 0x73, 0x18, 0x02, 0x20, 0x03, 0x28, 0x09, 0x52, 0x04,
	0x6d, 0x73, 0x67, 0x73, 0x22, 0xad, 0x02, 0x0a, 0x0e, 0x53, 0x74, 0x61, 0x74, 0x75, 0x73, 0x43,
	0x6f, 0x64, 0x65, 0x45, 0x6e, 0x75, 0x6d, 0x12, 0x0b, 0x0a, 0x07, 0x53, 0x55, 0x43, 0x43, 0x45,
	0x53, 0x53, 0x10, 0x00, 0x12, 0x1d, 0x0a, 0x19, 0x45, 0x52, 0x52, 0x4f, 0x52, 0x5f, 0x55, 0x4e,
	0x53, 0x55, 0x50, 0x50, 0x4f, 0x52, 0x54, 0x45, 0x44, 0x5f, 0x56, 0x45, 0x52, 0x53, 0x49, 0x4f,
	0x4e, 0x10, 0x01, 0x12, 0x1c, 0x0a, 0x18, 0x45, 0x52, 0x52, 0x4f, 0x52, 0x5f, 0x55, 0x4e, 0x45,
	0x58, 0x50, 0x45, 0x43, 0x54, 0x45, 0x44, 0x5f, 0x50, 0x41, 0x59, 0x4c, 0x4f, 0x41, 0x44, 0x10,
	0x02, 0x12, 0x11, 0x0a, 0x0d, 0x45, 0x52, 0x52, 0x4f, 0x52, 0x5f, 0x47, 0x45, 0x4e, 0x45, 0x52,
	0x49, 0x43, 0x10, 0x03, 0x12, 0x10, 0x0a, 0x0c, 0x45, 0x52, 0x52, 0x4f, 0x52, 0x5f, 0x44, 0x45,
	0x43, 0x4f, 0x44, 0x45, 0x10, 0x04, 0x12, 0x1d, 0x0a, 0x19, 0x45, 0x52, 0x52, 0x4f, 0x52, 0x5f,
	0x57, 0x52, 0x4f, 0x4e, 0x47, 0x5f, 0x41, 0x47, 0x45, 0x4e, 0x54, 0x5f, 0x41, 0x44, 0x44, 0x52,
	0x45, 0x53, 0x53, 0x10, 0x0a, 0x12, 0x1a, 0x0a, 0x16, 0x45, 0x52, 0x52, 0x4f, 0x52, 0x5f, 0x57,
	0x52, 0x4f, 0x4e, 0x47, 0x5f, 0x50, 0x55, 0x42, 0x4c, 0x49, 0x43, 0x5f, 0x4b, 0x45, 0x59, 0x10,
	0x0b, 0x12, 0x17, 0x0a, 0x13, 0x45, 0x52, 0x52, 0x4f, 0x52, 0x5f, 0x49, 0x4e, 0x56, 0x41, 0x4c,
	0x49, 0x44, 0x5f, 0x50, 0x52, 0x4f, 0x4f, 0x46, 0x10, 0x0c, 0x12, 0x1c, 0x0a, 0x18, 0x45, 0x52,
	0x52, 0x4f, 0x52, 0x5f, 0x55, 0x4e, 0x53, 0x55, 0x50, 0x50, 0x4f, 0x52, 0x54, 0x45, 0x44, 0x5f,
	0x4c, 0x45, 0x44, 0x47, 0x45, 0x52, 0x10, 0x0d, 0x12, 0x1f, 0x0a, 0x1b, 0x45, 0x52, 0x52, 0x4f,
	0x52, 0x5f, 0x55, 0x4e, 0x4b, 0x4e, 0x4f, 0x57, 0x4e, 0x5f, 0x41, 0x47, 0x45, 0x4e, 0x54, 0x5f,
	0x41, 0x44, 0x44, 0x52, 0x45, 0x53, 0x53, 0x10, 0x14, 0x12, 0x19, 0x0a, 0x15, 0x45, 0x52, 0x52,
	0x4f, 0x52, 0x5f, 0x41, 0x47, 0x45, 0x4e, 0x54, 0x5f, 0x4e, 0x4f, 0x54, 0x5f, 0x52, 0x45, 0x41,
	0x44, 0x59, 0x10, 0x15, 0x1a, 0x5b, 0x0a, 0x15, 0x52, 0x65, 0x67, 0x69, 0x73, 0x74, 0x65, 0x72,
	0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69, 0x76, 0x65, 0x12, 0x42, 0x0a,
	0x06, 0x72, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x18, 0x01, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x2a, 0x2e,
	0x61, 0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30,
	0x5f, 0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x2e, 0x41, 0x67,
	0x65, 0x6e, 0x74, 0x52, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x52, 0x06, 0x72, 0x65, 0x63, 0x6f, 0x72,
	0x64, 0x1a, 0x42, 0x0a, 0x1b, 0x4c, 0x6f, 0x6f, 0x6b, 0x75, 0x70, 0x5f, 0x52, 0x65, 0x71, 0x75,
	0x65, 0x73, 0x74, 0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69, 0x76, 0x65,
	0x12, 0x23, 0x0a, 0x0d, 0x61, 0x67, 0x65, 0x6e, 0x74, 0x5f, 0x61, 0x64, 0x64, 0x72, 0x65, 0x73,
	0x73, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09, 0x52, 0x0c, 0x61, 0x67, 0x65, 0x6e, 0x74, 0x41, 0x64,
	0x64, 0x72, 0x65, 0x73, 0x73, 0x1a, 0x62, 0x0a, 0x1c, 0x4c, 0x6f, 0x6f, 0x6b, 0x75, 0x70, 0x5f,
	0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d,
	0x61, 0x74, 0x69, 0x76, 0x65, 0x12, 0x42, 0x0a, 0x06, 0x72, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x18,
	0x01, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x2a, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e,
	0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f, 0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65,
	0x73, 0x73, 0x61, 0x67, 0x65, 0x2e, 0x41, 0x67, 0x65, 0x6e, 0x74, 0x52, 0x65, 0x63, 0x6f, 0x72,
	0x64, 0x52, 0x06, 0x72, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x1a, 0x7b, 0x0a, 0x19, 0x41, 0x65, 0x61,
	0x5f, 0x45, 0x6e, 0x76, 0x65, 0x6c, 0x6f, 0x70, 0x65, 0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72,
	0x6d, 0x61, 0x74, 0x69, 0x76, 0x65, 0x12, 0x1a, 0x0a, 0x08, 0x65, 0x6e, 0x76, 0x65, 0x6c, 0x6f,
	0x70, 0x65, 0x18, 0x01, 0x20, 0x01, 0x28, 0x0c, 0x52, 0x08, 0x65, 0x6e, 0x76, 0x65, 0x6c, 0x6f,
	0x70, 0x65, 0x12, 0x42, 0x0a, 0x06, 0x72, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x18, 0x02, 0x20, 0x01,
	0x28, 0x0b, 0x32, 0x2a, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e,
	0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f, 0x30, 0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61,
	0x67, 0x65, 0x2e, 0x41, 0x67, 0x65, 0x6e, 0x74, 0x52, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x52, 0x06,
	0x72, 0x65, 0x63, 0x6f, 0x72, 0x64, 0x1a, 0x54, 0x0a, 0x13, 0x53, 0x74, 0x61, 0x74, 0x75, 0x73,
	0x5f, 0x50, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69, 0x76, 0x65, 0x12, 0x3d, 0x0a,
	0x04, 0x62, 0x6f, 0x64, 0x79, 0x18, 0x01, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x29, 0x2e, 0x61, 0x65,
	0x61, 0x2e, 0x61, 0x65, 0x61, 0x2e, 0x61, 0x63, 0x6e, 0x2e, 0x76, 0x31, 0x5f, 0x30, 0x5f, 0x30,
	0x2e, 0x41, 0x63, 0x6e, 0x4d, 0x65, 0x73, 0x73, 0x61, 0x67, 0x65, 0x2e, 0x53, 0x74, 0x61, 0x74,
	0x75, 0x73, 0x42, 0x6f, 0x64, 0x79, 0x52, 0x04, 0x62, 0x6f, 0x64, 0x79, 0x42, 0x0e, 0x0a, 0x0c,
	0x70, 0x65, 0x72, 0x66, 0x6f, 0x72, 0x6d, 0x61, 0x74, 0x69, 0x76, 0x65, 0x62, 0x06, 0x70, 0x72,
	0x6f, 0x74, 0x6f, 0x33,
}

var (
	file_acn_proto_rawDescOnce sync.Once
	file_acn_proto_rawDescData = file_acn_proto_rawDesc
)

func file_acn_proto_rawDescGZIP() []byte {
	file_acn_proto_rawDescOnce.Do(func() {
		file_acn_proto_rawDescData = protoimpl.X.CompressGZIP(file_acn_proto_rawDescData)
	})
	return file_acn_proto_rawDescData
}

var file_acn_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_acn_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_acn_proto_goTypes = []interface{}{
	(AcnMessage_StatusBody_StatusCodeEnum)(0),       // 0: aea.aea.acn.v1_0_0.AcnMessage.StatusBody.StatusCodeEnum
	(*AcnMessage)(nil),                              // 1: aea.aea.acn.v1_0_0.AcnMessage
	(*AcnMessage_AgentRecord)(nil),                  // 2: aea.aea.acn.v1_0_0.AcnMessage.AgentRecord
	(*AcnMessage_StatusBody)(nil),                   // 3: aea.aea.acn.v1_0_0.AcnMessage.StatusBody
	(*AcnMessage_Register_Performative)(nil),        // 4: aea.aea.acn.v1_0_0.AcnMessage.Register_Performative
	(*AcnMessage_Lookup_Request_Performative)(nil),  // 5: aea.aea.acn.v1_0_0.AcnMessage.Lookup_Request_Performative
	(*AcnMessage_Lookup_Response_Performative)(nil), // 6: aea.aea.acn.v1_0_0.AcnMessage.Lookup_Response_Performative
	(*AcnMessage_Aea_Envelope_Performative)(nil),    // 7: aea.aea.acn.v1_0_0.AcnMessage.Aea_Envelope_Performative
	(*AcnMessage_Status_Performative)(nil),          // 8: aea.aea.acn.v1_0_0.AcnMessage.Status_Performative
}
var file_acn_proto_depIdxs = []int32{
	7,  // 0: aea.aea.acn.v1_0_0.AcnMessage.aea_envelope:type_name -> aea.aea.acn.v1_0_0.AcnMessage.Aea_Envelope_Performative
	5,  // 1: aea.aea.acn.v1_0_0.AcnMessage.lookup_request:type_name -> aea.aea.acn.v1_0_0.AcnMessage.Lookup_Request_Performative
	6,  // 2: aea.aea.acn.v1_0_0.AcnMessage.lookup_response:type_name -> aea.aea.acn.v1_0_0.AcnMessage.Lookup_Response_Performative
	4,  // 3: aea.aea.acn.v1_0_0.AcnMessage.register:type_name -> aea.aea.acn.v1_0_0.AcnMessage.Register_Performative
	8,  // 4: aea.aea.acn.v1_0_0.AcnMessage.status:type_name -> aea.aea.acn.v1_0_0.AcnMessage.Status_Performative
	0,  // 5: aea.aea.acn.v1_0_0.AcnMessage.StatusBody.code:type_name -> aea.aea.acn.v1_0_0.AcnMessage.StatusBody.StatusCodeEnum
	2,  // 6: aea.aea.acn.v1_0_0.AcnMessage.Register_Performative.record:type_name -> aea.aea.acn.v1_0_0.AcnMessage.AgentRecord
	2,  // 7: aea.aea.acn.v1_0_0.AcnMessage.Lookup_Response_Performative.record:type_name -> aea.aea.acn.v1_0_0.AcnMessage.AgentRecord
	2,  // 8: aea.aea.acn.v1_0_0.AcnMessage.Aea_Envelope_Performative.record:type_name -> aea.aea.acn.v1_0_0.AcnMessage.AgentRecord
	3,  // 9: aea.aea.acn.v1_0_0.AcnMessage.Status_Performative.body:type_name -> aea.aea.acn.v1_0_0.AcnMessage.StatusBody
	10, // [10:10] is the sub-list for method output_type
	10, // [10:10] is the sub-list for method input_type
	10, // [10:10] is the sub-list for extension type_name
	10, // [10:10] is the sub-list for extension extendee
	0,  // [0:10] is the sub-list for field type_name
}

func init() { file_acn_proto_init() }
func file_acn_proto_init() {
	if File_acn_proto != nil {
		return
	}
	if !protoimpl.UnsafeEnabled {
		file_acn_proto_msgTypes[0].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_acn_proto_msgTypes[1].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage_AgentRecord); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_acn_proto_msgTypes[2].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage_StatusBody); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_acn_proto_msgTypes[3].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage_Register_Performative); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_acn_proto_msgTypes[4].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage_Lookup_Request_Performative); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_acn_proto_msgTypes[5].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage_Lookup_Response_Performative); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_acn_proto_msgTypes[6].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage_Aea_Envelope_Performative); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_acn_proto_msgTypes[7].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*AcnMessage_Status_Performative); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
	}
	file_acn_proto_msgTypes[0].OneofWrappers = []interface{}{
		(*AcnMessage_AeaEnvelope)(nil),
		(*AcnMessage_LookupRequest)(nil),
		(*AcnMessage_LookupResponse)(nil),
		(*AcnMessage_Register)(nil),
		(*AcnMessage_Status)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: file_acn_proto_rawDesc,
			NumEnums:      1,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_acn_proto_goTypes,
		DependencyIndexes: file_acn_proto_depIdxs,
		EnumInfos:         file_acn_proto_enumTypes,
		MessageInfos:      file_acn_proto_msgTypes,
	}.Build()
	File_acn_proto = out.File
	file_acn_proto_rawDesc = nil
	file_acn_proto_goTypes = nil
	file_acn_proto_depIdxs = nil
}
