# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Profiler blacklisted types."""

PROFILER_TYPE_BLACK_LIST = {
    "ABCMeta",
    "Add",
    "AddressFamily",
    "AddressInfo",
    "AlertDescription",
    "And",
    "AugLoad",
    "AugStore",
    "BitAnd",
    "BitOr",
    "BitXor",
    "BufferedReader",
    "BufferedWriter",
    "CachedProperty",
    "CodecInfo",
    "Condition",
    "Counter",
    "Del",
    "Div",
    "DynamicClassAttribute",
    "Enum",
    "EnumMeta",
    "Eq",
    "Event",
    "ExtensionFileLoader",
    "FQP_corresponding_FQ_class",
    "FileFinder",
    "FileIO",
    "FloorDiv",
    "Formatter",
    "ForwardRef",
    "Gt",
    "GtE",
    "Handlers",
    "In",
    "IncrementalDecoder",
    "IncrementalEncoder",
    "Invert",
    "Is",
    "IsNot",
    "LShift",
    "Literal",
    "Load",
    "Logger",
    "Lt",
    "LtE",
    "Manager",
    "Marker",
    "MatMult",
    "MessageDescriptor",
    "Mod",
    "ModuleSpec",
    "MovedAttribute",
    "MsgFlag",
    "Mult",
    "NamedTupleMeta",
    "Not",
    "NotEq",
    "NotIn",
    "Op",
    "Options",
    "Or",
    "OrderedDict",
    "Param",
    "Parameter",
    "PercentStyle",
    "PlaceHolder",
    "Pow",
    "Purpose",
    "Quitter",
    "RShift",
    "Regex",
    "RegexFlag",
    "Repr",
    "Requirement",
    "RootLogger",
    "SSLErrorNumber",
    "Sigmasks",
    "Signals",
    "SimpleNamespace",
    "SocketKind",
    "SourceFileLoader",
    "Specifier",
    "SpecifierSet",
    "Store",
    "Sub",
    "TLSVersion",
    "TextIOWrapper",
    "TypeVar",
    "UAdd",
    "USub",
    "Value",
    "Variable",
    "VerifyFlags",
    "VerifyMode",
    "WeakSet",
    "WeakValueDictionary",
    "_ASN1Object",
    "_Environ",
    "_Finder",
    "_GenericAlias",
    "_Helper",
    "_MainThread",
    "_NamedIntConstant",
    "_NamespaceLoader",
    "_NamespacePath",
    "_ParameterKind",
    "_Printer",
    "_ProtocolMeta",
    "_RunningLoop",
    "_SSLMethod",
    "_SendfileMode",
    "_SpecialForm",
    "_StderrHandler",
    "_TLSAlertType",
    "_TLSContentType",
    "_TLSMessageType",
    "_TemplateMetaclass",
    "_TypedDictMeta",
    "_VariadicGenericAlias",
    "_lru_cache_wrapper",
    "_ttinfo",
    "_tuplegetter",
    "builtin_function_or_method",
    "cell",
    "classmethod",
    "classmethod_descriptor",
    "count",
    "curry",
    "cython_function_or_method",
    "date",
    "datetime",
    "deque",
    "dict",
    "frozenset",
    "function",
    "fused_cython_function",
    "getset_descriptor",
    "list",
    "mappingproxy",
    "member_descriptor",
    "method",
    "method-wrapper",
    "method_descriptor",
    "module",
    "partial",
    "property",
    "set",
    "slice",
    "staticmethod",
    "time",
    "timedelta",
    "timezone",
    "tuple",
    "type",
    "tzfile",
    "vectorize",
    "weakref",
    "wrapper_descriptor",
}
