# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for project helper tools."""

import os

from aea.manager.helpers import AEAProject

PROJECT_CONTENT = [
    "contracts",
    "connections",
    "skills",
    "aea-config.yaml",
    "vendor",
    "protocols",
]


def test_aea_project():
    """Test AEAProject context manager"""

    cwd = os.getcwd()
    with AEAProject():
        project_wd = os.getcwd()
        assert cwd != project_wd
        assert set(os.listdir(project_wd)) == set(PROJECT_CONTENT)

    assert cwd == os.getcwd()
    assert not os.path.exists(project_wd)
